/*
 * Copyright 2012 ETH Zuerich, CISD
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ch.ethz.cisd.hotdeploy;

import java.io.File;

/**
 * @author Bernd Rinn
 */
final class PluginDesc
{
    /** The plugin class. */
    private final Class<?> pluginClass;

    /** The plugin jar archive. */
    private final File pluginArchive;

    /** The time of last modification of the pluginArchive that the pluginClass corresponds to. */
    private final long lastModified;

    PluginDesc(Class<?> pluginClass, File pluginArchive, long lastModified)
    {
        this.pluginClass = pluginClass;
        this.pluginArchive = pluginArchive;
        this.lastModified = lastModified;
    }

    /**
     * Returns the plugin class.
     */
    Class<?> getPluginClass()
    {
        return pluginClass;
    }

    /**
     * Returns the name of the plugin class.
     */
    String getPluginClassname()
    {
        return pluginClass.getName();
    }

    /**
     * Get plugin information for a plugin by its classname.
     * 
     * @return the plugin information or {@link PluginDesc} if plugin has no plugin information.
     */
    PluginInfo getPluginInfo()
    {
        return pluginClass.getAnnotation(PluginInfo.class);
    }

    /**
     * Returns the name of the plugin. If the plugin has a {@link PluginInfo}, it will be
     * {@link PluginInfo#name()}, otherwise it will be the name of the plugin class.
     */
    String getPluginName()
    {
        final PluginInfo info = pluginClass.getAnnotation(PluginInfo.class);
        if (info != null)
        {
            return info.name();
        } else
        {
            return pluginClass.getName();
        }
    }

    /**
     * Returns the type of the plugin. If the plugin has a {@link PluginInfo}, it will be
     * {@link PluginInfo#pluginType()}, otherwise it will be the plugin class itself.
     */
    Class<?> getPluginType()
    {
        final PluginInfo info = pluginClass.getAnnotation(PluginInfo.class);
        if (info != null)
        {
            return info.pluginType();
        } else
        {
            return getPluginClass();
        }
    }

    /**
     * Returns <code>true</code> if the plugin class is implementing the given
     * <var>workerInterface</var>.
     */
    boolean isImplementing(Class<?> workerInterface)
    {
        return workerInterface.isAssignableFrom(pluginClass);
    }

    /**
     * Returns the jar file that the plugin class was loaded from.
     */
    File getPluginArchive()
    {
        return pluginArchive;
    }

    /**
     * Returns the directory that the plugin was found in.
     */
    File getPluginDirectory()
    {
        return pluginArchive.getParentFile();
    }

    /**
     * Returns the time of last modification of the plugin archive that the plugin class corresponds
     * to.
     */
    long getLastModified()
    {
        return lastModified;
    }

    /**
     * Returns <code>true</code> if the plugin archive has changed since this plugin class has been
     * loaded.
     */
    boolean hasPluginArchiveChanged()
    {
        return pluginArchive.lastModified() != lastModified;
    }
}
