/*
 * Copyright 2005-2009 Lukas Blunschi
 * Copyright 2012 ETH Zuerich, CISD
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ch.ethz.cisd.hotdeploy;

import java.io.File;

public final class PluginEvent
{
    public enum PluginEventType
    {
        REGISTER_NEW_PLUGIN, UPDATE_PLUGIN, UNREGISTER_PLUGIN
    }

    private final PluginEventType eventType;

    private final Class<?> plugin;

    private final File pluginArchive;

    public PluginEvent(PluginEventType eventType, Class<?> plugin, File pluginArchive)
    {
        this.eventType = eventType;
        this.plugin = plugin;
        this.pluginArchive = pluginArchive;
    }

    /**
     * @return The type of plugin event.
     */
    public PluginEventType getEventType()
    {
        return eventType;
    }

    /**
     * @return The plugin associated with this event, or <code>null</code>, if this is a shutdown
     *         event.
     */
    public Class<?> getPlugin()
    {
        return plugin;
    }

    /**
     * @return The class name of the plugin, or <code>null</code>, if this is a shutdown event.
     */
    public String getPluginClassname()
    {
        return (plugin == null) ? null : plugin.getName();
    }

    /**
     * @return The name of the plugin, or <code>null</code>, if this is a shutdown event.. If the
     *         plugin has a {@link PluginInfo}, it will be {@link PluginInfo#name()}, otherwise it
     *         will be the name of the plugin class.
     */
    public String getPluginName()
    {
        final PluginInfo info = plugin.getAnnotation(PluginInfo.class);
        if (info != null)
        {
            return info.name();
        } else
        {
            return (plugin == null) ? null : plugin.getName();
        }
    }

    /**
     * @return The archive file of the plugin, if known and <code>null</code> else.
     */
    public File getPluginArchive()
    {
        return pluginArchive;
    }

    /**
     * @return The type of the plugin. If the plugin has a {@link PluginInfo}, it will be
     *         {@link PluginInfo#pluginType()}, otherwise it will be the plugin class itself.
     */
    public Class<?> getPluginType()
    {
        final PluginInfo info = plugin.getAnnotation(PluginInfo.class);
        if (info != null)
        {
            return info.pluginType();
        } else
        {
            return plugin;
        }
    }

    /**
     * @return <code>true</code>, if the plugin implements the <var>workerInterface</var>.
     */
    public boolean isPluginImplementingInterface(Class<?> workerInterface)
    {
        return workerInterface.isAssignableFrom(plugin);
    }

    /**
     * @return <code>true</code> if this is a registration or update event (either new plugin or
     *         update of existing plugin).
     */
    public boolean isRegistrationOrUpdateEvent()
    {
        return eventType != PluginEventType.UNREGISTER_PLUGIN;
    }

    /**
     * @return <code>true</code> if this is an unregistration / unloading event.
     */
    public boolean isUnregistrationEvent()
    {
        return eventType == PluginEventType.UNREGISTER_PLUGIN;
    }

    /**
     * @return <code>true</code> if this is a shutdown event or <code>false</code> otherwise.
     */
    public boolean isShutdownEvent()
    {
        return plugin == null;
    }

}
