/*
 * Copyright 2012 ETH Zuerich, CISD
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ch.ethz.cisd.hotdeploy;

/**
 * A factory to create instances of a given plugin.
 * 
 * @author Bernd Rinn
 */
public final class PluginFactory<T> extends PluginInfoProvider
{
    private final Class<T> pluginType;

    private final Object[] defaultConstructorArgs;

    /**
     * Creates a plugin factory for a given plugin. The plugin is <i>not</i> checked for
     * availability when creating the factory.
     * 
     * @param pluginName The name of the plugin or the plugin class to create.
     * @param isClassName If <code>true</code>, <var>pluginName</var> will be considered a class
     *            name, else it is considered a plugin name.
     * @param defaultConstructorArgs The default arguments of the plugin's constructor (can be
     *            overridden).
     */
    public PluginFactory(PluginContainer container, String pluginName, boolean isClassName,
            Class<T> pluginType, Object[] defaultConstructorArgs)
    {
        super(container, pluginName, isClassName);
        this.pluginType = pluginType;
        this.defaultConstructorArgs = defaultConstructorArgs;
    }

    /**
     * Creates a new instance of the plugin.
     * 
     * @param constructorArgs If not empty, use these as the constructor arguments, otherwise, use
     *            the default constructor arguments provided to the constructor of the factory.
     */
    public T createInstance(Object... constructorArgs)
    {
        if (constructorArgs.length > 0)
        {
            return createInstanceWithConstructorArguments(constructorArgs);
        } else
        {
            return createInstanceWithDefaultConstructorArguments();

        }
    }

    /**
     * Creates a new instance of the plugin with the default constructor arguments provided to the
     * constructor of the factory.
     */
    public T createInstanceWithDefaultConstructorArguments()
    {
        return isClassName ? container.tryCreatePluginByClassname(pluginName, pluginType,
                defaultConstructorArgs) : container.tryCreatePlugin(pluginName, pluginType,
                defaultConstructorArgs);
    }

    /**
     * Creates a new instance of the plugin.
     * 
     * @param constructorArgs The arguments of the constructors of the plugin.
     */
    public T createInstanceWithConstructorArguments(Object... constructorArgs)
    {
        return isClassName ? container.tryCreatePluginByClassname(pluginName, pluginType,
                constructorArgs) : container.tryCreatePlugin(pluginName, pluginType, constructorArgs);
    }

}
