/*
 * Copyright 2012 ETH Zuerich, CISD
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ch.ethz.cisd.hotdeploy;

import java.io.File;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * A class for holding the pseudo-singleton of a plugin. The pseudo-singleton is re-created each
 * time a new version of the plugin becomes available and is <code>null</code>ified when the plugin
 * is unregistered.
 * <p>
 * All users of this plugin holder who cache an instance of the pseudo-singleton plugin are supposed
 * to register as a {@link PluginEventListener} of this holder and release all references of the
 * former plugin when notified of a change.
 * 
 * @author Bernd Rinn
 */
public class PluginHolder<T> extends PluginInfoProvider
{
    private static final Log log = LogFactory.getLog(PluginHolder.class);

    private final Class<T> workerInterface;

    private final Object[] constructorArgs;

    private T instance;

    /**
     * Creates a plugin holder for the given plugin. The plugin singleton instance is created in the
     * constructor.
     * 
     * @param pluginName The name of the plugin to hold.
     * @param pluginInterface The interface to cast the plugin to.
     * @param constructorArgs The arguments of the plugin's constructor.
     */
    public PluginHolder(final PluginContainer container, final String pluginName,
            final Class<T> pluginInterface, final Object... constructorArgs)
    {
        this(container, pluginName, false, pluginInterface, constructorArgs);
    }

    /**
     * Creates a plugin holder for the given plugin. The plugin singleton instance is created in the
     * constructor.
     * 
     * @param pluginName The name of the plugin or the plugin class to hold.
     * @param isClassName If <code>true</code>, <var>pluginName</var> will be considered a class
     *            name, else it is considered a plugin name.
     * @param workerInterface The interface to cast the plugin to.
     * @param constructorArgs The arguments of the plugin's constructor.
     */
    public PluginHolder(final PluginContainer container, final String pluginName,
            boolean isClassName, final Class<T> workerInterface, final Object... constructorArgs)
    {
        super(container, pluginName, isClassName);
        this.workerInterface = workerInterface;
        this.constructorArgs = constructorArgs;
        this.instance =
                container.tryCreatePluginByClassname(pluginName, workerInterface, constructorArgs);
    }

    @Override
    protected void processEvent(PluginEvent event)
    {
        if (event.isRegistrationOrUpdateEvent())
        {
            final T newInstance =
                    isClassName ? container.tryCreatePluginByClassname(pluginName, workerInterface,
                            constructorArgs) : container.tryCreatePlugin(pluginName, workerInterface,
                            constructorArgs);
            if (log.isDebugEnabled())
            {
                log.debug("Set new plugin instance for plugin " + pluginName);
            }
            setInstance(newInstance, event);
        } else
        {
            if (log.isDebugEnabled())
            {
                log.debug("Remove plugin instance for plugin " + pluginName);
            }
            setInstance(null, event);
        }
        super.processEvent(event);
    }

    synchronized void setInstance(T newInstance, PluginEvent event)
    {
        this.instance = newInstance;
    }

    /**
     * Returns the (current) instance of this plugin, or <code>null</code>, if there is no instance
     * for it.
     */
    public synchronized T tryGetInstance()
    {
        return instance;
    }

    /**
     * Returns the given <var>resource</var> in the jar archive of this plugin, or <code>null</code>
     * , if it cannot be found.
     */
    public byte[] tryGetResource(String resource)
    {
        final File pluginArchive =
                isClassName ? container.tryGetPluginArchiveByClassname(pluginName) : container
                        .tryGetPluginArchive(pluginName);
        return container.tryGetPluginResource(pluginArchive, resource);
    }

    /**
     * Refresh plugins now.
     * 
     * @param synchronous if <code>true</code>, wait for the refresh cycle to finish before
     *            returning.
     */
    public void refresh(boolean synchronous)
    {
        container.refresh(synchronous);
    }
}
