/*
 * Copyright 2012 ETH Zuerich, CISD
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ch.ethz.cisd.hotdeploy;

import java.io.Closeable;
import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import ch.systemsx.cisd.base.exceptions.CheckedExceptionTunnel;

/**
 * A class for loading a plugin test-wise. No automatic reloading will happen when the plugin
 * archive changes. When done with the plugin, call {@link #close()} to remove the classloader and
 * all classes it has loaded.
 * <p>
 * <i>Plugin loading with this class is not thread-safe. Use it only in cases when exactly one
 * thread is in charge of plugin loading.</i>
 * 
 * @author Bernd Rinn
 */
public class PluginTester implements Closeable
{
    private final PluginDeploymentDesc descriptor;
    
    private final Writer logWriter;

    /**
     * Creates the plugin tester and loads the plugin.
     * 
     * @param pluginArchive The jar archive to load the plugin from.
     * @param requirePluginInfo If <code>true</code>, only a class with {@link PluginInfo}
     *            annotation will be considered a valid archive, if <code>false</code>, also a class
     *            implementing the {@link Plugin} marker interface is considered a valid plugin.
     * @throws IllegalArgumentException If the <var>pluginArchive</var> does not contain a valid
     *             plugin.
     */
    public PluginTester(File pluginArchive, boolean requirePluginInfo)
    {
        this(pluginArchive, new StringWriter(), requirePluginInfo);
    }
    
    /**
     * Creates the plugin tester and loads the plugin.
     * 
     * @param pluginArchive The jar archive to load the plugin from.
     * @param logWriter The log writer to log error messages to.
     * @param requirePluginInfo If <code>true</code>, only a class with {@link PluginInfo}
     *            annotation will be considered a valid archive, if <code>false</code>, also a class
     *            implementing the {@link Plugin} marker interface is considered a valid plugin.
     * @throws IllegalArgumentException If the <var>pluginArchive</var> does not contain a valid
     *             plugin.
     */
    public PluginTester(File pluginArchive, Writer logWriter, boolean requirePluginInfo)
    {
        this.descriptor = new PluginDeploymentDesc(pluginArchive, requirePluginInfo);
        this.logWriter = logWriter;
        if (descriptor.isLoaded() == false)
        {
            throw new IllegalArgumentException("File " + pluginArchive
                    + " does not contain a valid plugin.");
        }
    }
    
    private void checkOpen() throws IllegalStateException
    {
        if (descriptor.isLoaded() == false)
        {
            throw new IllegalStateException("Plugin has been closed.");
        }
    }

    /**
     * Returns the error messages logged when trying to create a plugin.
     */
    public String getErrorMessages()
    {
        return logWriter.toString();
    }
    
    /**
     * Get the plugin class by its <var>classname</var>.
     * 
     * @return The plugin class.
     */
    public Class<?> getPluginClass()
    {
        checkOpen();
        return descriptor.getPluginClass();
    }

    /**
     * Creates a new instance of the plugin.
     * 
     * @param workerInterface The worker interface to cast the plugin to.
     * @return The plugin or <code>null</code> if not registered or name is <code>null</code>.
     * @throws IllegalArgumentException If the plugin has no constructor with these arguments.
     */
    public <T> T tryCreatePlugin(Class<T> workerInterface, Object... constructorArguments)
    {
        checkOpen();
        final Class<?> pluginClass = getPluginClass();
        try
        {
            return ReflectionUtils.create(workerInterface, pluginClass, constructorArguments);
        } catch (IllegalArgumentException ex)
        {
            appendToLog(ex.getMessage());
        } catch (ClassCastException ex)
        {
            appendToLog(ex.getMessage());
        }
        return null;
    }
    
    private void appendToLog(String msg)
    {
        try
        {
            if (logWriter.toString().isEmpty() == false)
            {
                logWriter.append('\n');
            }
            logWriter.append(msg);
        } catch (IOException ex)
        {
            throw CheckedExceptionTunnel.wrapIfNecessary(ex);
        }
    }

    /**
     * Returns <code>true</code> if the plugin has a constructor for the
     * <var>constructorArgumentTypes</var>.
     * 
     * @param constructorArgumentTypes The types of the constructor arguments. Empty for default
     *            constructor.
     */
    public boolean hasConstructor(Class<?>... constructorArgumentTypes)
    {
        checkOpen();
        final Class<?> pluginClass = getPluginClass();
        if (pluginClass == null)
        {
            return false;
        }
        return ReflectionUtils.tryGetConstructor(pluginClass, constructorArgumentTypes) != null;
    }

    /**
     * Get the name of the plugin class.
     * 
     * @return The name of the plugin class.
     */
    public String getPluginClassname()
    {
        checkOpen();
        return descriptor.getPluginDesc().getPluginClassname();
    }

    /**
     * Get the name of the plugin.
     * 
     * @return The name of the plugin.
     */
    public String getPluginName()
    {
        checkOpen();
        return descriptor.getPluginDesc().getPluginName();
    }

    /**
     * Get the plugin info object.
     * 
     * @return The plugin info object, or <code>null</code>, if this plugin has no plugin info
     *         object defined.
     */
    public PluginInfo tryGetPluginInfo()
    {
        checkOpen();
        return descriptor.getPluginDesc().getPluginInfo();
    }

    /**
     * Get the plugin archive that this tester has been created with.
     * 
     * @return The plugin archive file.
     */
    public File getPluginArchive()
    {
        checkOpen();
        return descriptor.getPluginArchive();
    }
    
    /**
     * Frees all classes and the classloader of this plugin.
     */
    @Override
    public void close()
    {
        descriptor.unload();
    }

    @Override
    protected void finalize() throws Throwable
    {
        close();
        super.finalize();
    }

}
