/*
 * Copyright 2012 ETH Zuerich, CISD
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ch.ethz.cisd.hotdeploy;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;

import ch.systemsx.cisd.base.exceptions.CheckedExceptionTunnel;

/**
 * Helper methods for reflection.
 * 
 * @author Christian Ribeaud
 * @author Bernd Rinn
 */
final class ReflectionUtils
{
    /**
     * Creates a new instance of a class specified by its fully-qualified name.
     * 
     * @param workerInterface Worker interface that <code>className</code> has to implement.
     * @param clazz Fully qualified class.
     * @param arguments Optional constructor arguments. If <code>(Object[])</code> is an empty
     *            array, then the default constructor will be used.
     * @return an instance of type <code>interface</code>.
     * @throw IllegalArgumentException If the class cannot be instantiated, e.g. because it is
     *        abstract or because it is lacking a constructor with the given arguments.
     * @throw ClassCastException If the instance cannot be cast to <var>workerInterface</var>.
     * @throw {@link RuntimeException} or any sub-class of it: rethrown if calling the constructor
     *        fails.
     */
    static <T, C> T create(final Class<T> workerInterface, final Class<C> clazz,
            final Object... arguments)
    {
        assert workerInterface != null : "Missing worker interface";
        assert clazz != null : "Missing class name";

        try
        {
            if (arguments == null || arguments.length == 0)
            {
                return cast(clazz.newInstance(), workerInterface);
            }
            final Class<?>[] classes = getClasses(arguments);
            final Constructor<T> constructor = tryGetConstructor(clazz, classes);
            if (constructor == null)
            {
                throw new IllegalArgumentException(String.format(
                        "Cannot instantiate class '%s' with given arguments '%s'"
                                + " (constructor not found).", clazz.getName(),
                        Arrays.asList(arguments)));
            }
            final T instance =
                    cast(constructor.newInstance(arguments), workerInterface);
            return instance;
        } catch (final InstantiationException ex)
        {
            throw new IllegalArgumentException(String.format(
                    "Cannot instantiate class '%s' with given arguments '%s'"
                            + " (class is non-instantiable).", clazz.getName(),
                    Arrays.asList(arguments)));
        } catch (final IllegalAccessException ex)
        {
            throw new IllegalArgumentException(String.format(
                    "Cannot instantiate class '%s' with given arguments '%s' "
                            + "(access violation).", clazz.getName(), Arrays.asList(arguments)));
        } catch (final InvocationTargetException ex)
        {
            throw CheckedExceptionTunnel.wrapIfNecessary(ex.getCause());
        }
    }

    @SuppressWarnings("unchecked")
    private static <T> T cast(final Object instance,
            final Class<T> workerInterface)
    {
        if (workerInterface.isInstance(instance) == false)
        {
            throw new ClassCastException(String.format(
                    "Class '%s' does not implement interface '%s'.", instance.getClass().getName(),
                    workerInterface.getName()));
        }
        return (T) instance;
    }

    static Class<?>[] getClasses(final Object... initargs)
    {
        final Class<?>[] classes = new Class<?>[initargs.length];
        int i = 0;
        for (final Object initarg : initargs)
        {
            classes[i++] = initarg.getClass();
        }
        return classes;
    }

    static <T> Constructor<T> tryGetConstructor(final Class<?> clazz, final Class<?>[] classes)
    {
        final Constructor<?>[] constructors = clazz.getConstructors();
        Constructor<?> returned = null;
        for (final Constructor<?> constructor : constructors)
        {
            final Class<?>[] parameterTypes = constructor.getParameterTypes();
            final int len = parameterTypes.length;
            if (len != classes.length)
            {
                continue;
            }
            boolean match = true;
            for (int i = 0; i < len; i++)
            {
                final Class<?> parameterType = parameterTypes[i];
                final Class<?> c = classes[i];
                match &= parameterType.equals(c) || parameterType.isAssignableFrom(c);
            }
            if (match)
            {
                returned = constructor;
            }
        }
        return toGenericType(returned);
    }

    @SuppressWarnings("unchecked")
    private static <T> Constructor<T> toGenericType(final Constructor<?> returned)
    {
        return (Constructor<T>) returned;
    }

    private ReflectionUtils()
    {
        // Cannot be instantiated.
    }

}
